
;+
; NAME:
;        IRUHXASCAN2SXT
; PURPOSE:
;        Calculate suncenter positions (in SXT pixel coordinates) by
;        combining IRU and HXA scan data.
;        This program should be called with only one orbit worth of data
;        at a time.
; CATEGORY:
; CALLING SEQUENCE:
;        sunc = iruhxascan2sxt(iru_i,iru_d,hxa_i,hir_d,hxa_d,hxa_scan)
;        sunc = iruhxascan2sxt(atr,atr.iru,atr,atr.iru,atr.hxa,hxa_scan)
;        sunc = iruhxascan2sxt(mk_timarr(ada_index,8,/struct), $
;                          reform(ada_data.iru,3,8*n_elements(ada_data)), $
;                          ada_index,gt_iru(ada_data.iru),gt_hxa(ada_data), $
;                          hxa_scan)
; INPUTS:
;        iru_i = index for iru_d. pnt, atr, or mk_timarr(ada_index,8,/struct)
;        iru_d = array(3,m). iru data, e.g., pnt.iru, reform(ada_data.iru,3,n8)
;        hxa_i = index for hxa_d and hir_d, e.g., pnt, atr, or ada_index
;        hir_d = array(3,n)iru data COTEMPORAL with hxa_d, e.g. pnt.iru
;        hxa_d = array(4,n). HXA data, e.g., pnt.hxa or gt_hxa(ada_data)
;        hxa_scan = vector of struct.  Straight output of rd_hxa for 1 orbit
; KEYWORDS (INPUT):
; OUTPUTS:
;        sunc = suncenter coordinates in SXT full resolution pixels for
;               all times in iru_i.  sunc(0,*) = x, sunc(1,*) = y
; KEYWORDS (OUTPUT):
;        code = vector.  Indicates the reliability of each sun center position.
;               1: plain S/C commanded value used, no dejittering with IRU.
;               2: S/C commanded value dejittered with IRU.
;               4: HXA value dejittered with IRU.  Standard result, ok.
;               8: HXA scan data result dejittered with IRU.
;               Notes: - S/C commanded values can be several pixels off.
;                      - HXA scan data is better than regular data since the
;                        degradation of the HXA detector (after fall 1997).
; COMMON BLOCKS:
;        None.
; SIDE EFFECTS:
; RESTRICTIONS:
;        This program should be called with only one orbit worth of data
;        at a time.
; PROCEDURE:
;        The HXA detectors provide absolute suncenter positions, but with a
;        relatively coarse resolution.  The gyros (IRU) have better resolution
;        but poor longterm stability.  The program fits the gyro data onto
;        the HXA data to obtain the absolute accuracy of the HXA, and the
;        high resolution of the gyros.  The gyros show a drift which can
;        assumed to be linear for time intervals of up to about 10 minutes.
;        If the covered time period is longer, the program splits the data
;        into shorter intervals and performs a linear fit for each interval.
; MODIFICATION HISTORY:
;        19-Jul-93 (JPW)  From get_suncenter.
;        27-Sep-93 (JPW)  Added call to iru_s2y to correct for iru switch.
;        26-Sep-98 (JPW)  From iruhxa2sxt, adding code to use hxa scan data.
;                         Also made sure that IRU is not fitted to a mix of
;                         hxa and commanded pointing (caused spurious drift).
;        16-Mar-99 (JPW)  Fixed bug (in some cases, scan data was selected
;                         even if less than two data points were available).
;         8-May-03 (JPW)  Use scan data only on or after 28-sep-97.
;        19-May-03 (JPW)  Use scan data only on or after 2-jun-96 and
;                         add fudge constant -0.3 to hxa scan data
;                         [modified lines marked with ;03;]
;        29-May-03 (JPW)  Extend search for scan data after 26-sep-00
;                         [modified lines marked with ;03;]
;-

function iruhxascan2sxt,iru_i,iru_d0,hxa_i,hir_d0,hxa_d,hxa_s,code=code

; various constants
fiscaday = 6363    ;03; = gt_day('2-jun-96') , first day to use hxa scan data
fibliday = 7940    ;03; = gt_day('26-sep-2000') , first day of bad limb data
hxyfudge = -0.3    ;03; fudge constant to match limb and scan data on fiscaday

tstep = 210        ; time step (in sec) used for recalc. linear IRU fit
                   ; Important: 5*tstep must be greater than 1024

; make copies of iru_d, hir_d.  Copies will be modified by program
iru_d = iru_d0
hir_d = hir_d0

; find the good iru values in hir_d : pointer wwp (wwp is always valid)
iru_filt,hxa_i,hir_d,w_good=wwp
; transform IRU-S to IRU-Y values (if necessary)
hir_d = iru_s2y(hxa_i,hir_d)

; fix the bad iru values in iru_d by interpolation.
iru_filt,iru_i,iru_d
; transform IRU-S to IRU-Y values (if necessary)
iru_d = iru_s2y(iru_i,iru_d)

; get commanded suncenter positions as a fallback solution
dtt = anytim2ints(hxa_i(wwp))
dir = hir_d(*,wwp)
dxy = sxt_cmd_pnt(dtt)

; reconstruct the hxa hidden limbs and transform into sxt suncenter coord.
cxy = hxa2hxaxy(dtt,hxa_d(*,wwp),code=hcod)
cxy = hxaxy2sxt(dtt,cxy)
wcp = where(hcod gt 1)
if n_elements(wcp) ge 2 then begin
   ctt = dtt(wcp)
   cir = dir(*,wcp)
   cxy = cxy(*,wcp)
endif else ctt = 0

; calculate suncenter coordinates from hxa scan data
; conditional, so that we can run without it, if necessary
btt = 0
if n_elements(hxa_s) gt 1 and gt_day(iru_i(0)) ge fiscaday then begin  ;03;
   bxy = hxascan2hxaxy(hxa_s,btt,cod=scod)
   bxy = bxy + hxyfudge                                                ;03;
   bxy = hxaxy2sxt(btt,bxy)
   wwbi = tim2dset(dtt,btt,delta_sec=dbi)
   bir = dir(*,wwbi)
   wcp = where(dbi eq 0 and scod eq 1)
   if n_elements(wcp) ge 2 then begin
      btt = btt(wcp)
      bir = bir(*,wcp)
      bxy = bxy(*,wcp)
   endif else btt = 0     ; else part added on 99/3/16 (bug fix)
endif
; flag set extends scan range                                        ;03;
if gt_day(iru_i(0)) ge fibliday then fibliflg = 1 else fibliflg = 0  ;03; 

n_i = n_elements(iru_i)
out = fltarr(2,n_i)                     ; output variable
code = intarr(n_i)                      ; output data quality code

list = bytarr(n_i)+1          ; list to check off data points
wlis = where(list)

repeat begin  ; loop through short time intervals for which gyros are linear

   ; search for the earliest time in the list
   dummy = min(int2secarr(iru_i(wlis)),wlo)
   wlo = wlis(wlo)

   tilo = anytim2ints(iru_i(wlo))
   tihi = anytim2ints(iru_i(wlo),off=tstep)
   wwi = sel_timrange(iru_i(wlis),tilo,tihi)
   wwi = wlis(wwi)
   list(wwi) = 0B                   ; cross off selected elements in list

   tclo = anytim2ints(iru_i(wlo),off=-tstep)
   tchi = anytim2ints(iru_i(wlo),off=2*tstep)
   
   att = 0    ; initially an element, becomes vector as soon as data found

   ; first check if we could use hxa scan data   
   if n_elements(btt) ge 2 then begin
      wwc = sel_timrange(btt,tclo,tchi,/between)
      ; if only one hxa scan data point found, try several larger intervals:
      ; (a) centered
      if n_elements(wwc) lt 2 and wwc(0) ge 0 then begin
         tblo = anytim2ints(iru_i(wlo),off=-2*tstep)
         tbhi = anytim2ints(iru_i(wlo),off=3*tstep)
         wwc = sel_timrange(btt,tblo,tbhi,/between)
      endif
      ; (b) off-center - early
      if n_elements(wwc) lt 2 and wwc(0) ge 0 then begin
         tblo = anytim2ints(iru_i(wlo),off=-3*tstep)
         tbhi = anytim2ints(iru_i(wlo),off=2*tstep)
         wwc = sel_timrange(btt,tblo,tbhi,/between)
      endif
      ; (c) off-center - late
      if n_elements(wwc) lt 2 and wwc(0) ge 0 then begin
         tblo = anytim2ints(iru_i(wlo),off=-tstep)
         tbhi = anytim2ints(iru_i(wlo),off=4*tstep)
         wwc = sel_timrange(btt,tblo,tbhi,/between)
      endif
      ;03; extend search interval after fibliday to avoid hxa limb data
      if fibliflg and n_elements(wwc) lt 2 then begin ;03;
         tblo = anytim2ints(iru_i(wlo),off=-4*tstep)  ;03;
         tbhi = anytim2ints(iru_i(wlo),off=5*tstep)   ;03;
         wwc = sel_timrange(btt,tblo,tbhi,/between)   ;03;
         ; if only one hxa scan data point found, try larger intervals: ;03;
         ; (a) off-center - early                            ;03;
         if n_elements(wwc) lt 2 and wwc(0) ge 0 then begin  ;03;
            tblo = anytim2ints(iru_i(wlo),off=-12*tstep)     ;03;
            tbhi = anytim2ints(iru_i(wlo),off=5*tstep)       ;03;
            wwc = sel_timrange(btt,tblo,tbhi,/between)       ;03;
         endif                                               ;03;
         ; (b) off-center - late                             ;03;
         if n_elements(wwc) lt 2 and wwc(0) ge 0 then begin  ;03;
            tblo = anytim2ints(iru_i(wlo),off=-4*tstep)      ;03;
            tbhi = anytim2ints(iru_i(wlo),off=13*tstep)      ;03;
            wwc = sel_timrange(btt,tblo,tbhi,/between)       ;03;
         endif                                               ;03;
      endif                                                  ;03;
      ; found enough data
      if n_elements(wwc) ge 2 then begin
         tmin = min(int2secarr(btt(wwc)),max=tmax)
         if (tmax-tmin) ge tstep/10 then begin    ; time interval long enough?
            att = btt(wwc)
            air = bir(*,wwc)
            axy = bxy(*,wwc)
            code(wwi) = 8                 ; code for IRU fitted HXA scan data
         endif
      endif
   endif

   ; if hxa scan data didn't work, try hxa address data
   if n_elements(att) lt 2 and n_elements(ctt) ge 2 then begin
      wwc = sel_timrange(ctt,tclo,tchi,/between)
      if n_elements(wwc) ge 2 then begin
         tmin = min(int2secarr(ctt(wwc)),max=tmax)
         if (tmax-tmin) ge tstep/10 then begin    ; time interval long enough?
            att = ctt(wwc)
            air = cir(*,wwc)
            axy = cxy(*,wwc)
            code(wwi) = 4                 ; code for IRU fitted HXA addr data
         endif
      endif
   endif

   ; if hxa didn't work at all, try dejittering commanded data
   if n_elements(att) lt 2 and n_elements(dtt) ge 2 then begin
      wwc = sel_timrange(dtt,tclo,tchi,/between)
      if n_elements(wwc) ge 2 then begin
         tmin = min(int2secarr(dtt(wwc)),max=tmax)
         if (tmax-tmin) ge tstep/10 then begin    ; time interval long enough?
            att = dtt(wwc)
            air = dir(*,wwc)
            axy = dxy(*,wwc)
            code(wwi) = 2                 ; code for IRU fitted S/C com. data
         endif
      endif
   endif

   ; do the fitting procedure

   if n_elements(att) ge 2 then begin
      out(*,wwi) = iruhxa_fit(iru_i(wwi),iru_d(*,wwi),att,air,axy,degree=2)
   endif else begin
      ; use plain S/C commanded value
      out(*,wwi) = sxt_cmd_pnt(iru_i(wwi))
      code(wwi) = 1
   endelse

   wlis = where(list)
endrep until wlis(0) lt 0                  ; loop until list empty

return,out
end

